import {
  Box,
  Text,
  Button,
  Grid,
  GridItem,
  Image,
  Input,
  Menu,
  MenuButton,
  MenuItem,
  MenuList,
  Radio,
  Stack,
  useRadio,
  useRadioGroup,
  UseRadioGroupReturn,
} from "@chakra-ui/react";
import { ChevronDown } from "@feedloop/icon";
import React from "react";
import {
  defineComponent,
  EditorContext,
  useSourceHooks,
  useEvaluate,
  ListItemVariables,
  RuntimeContext,
  useRuntimeInput,
} from "@qorebase/app-component";
import {
  InsightSchema,
  Row,
  TableSchema,
  ViewSchema,
} from "@qorebase/app-runtime/src/interfaces";
import { fallbackImg } from "@qorebase/app-runtime/src/utils/fallbackImage";

const Option = (
  props: {
    item: Row;
    dataValue: string;
    onChange: (value: string) => void;
    setSelected: React.Dispatch<React.SetStateAction<string>>;
    search?: string;
  } & typeof DropdownListComponent["Component"]["defaultProps"]
) => {
  const value = useEvaluate(
    props.properties.itemValue ||
      String(props.item.id) ||
      String(props.source.itemId)
  );
  const label = useEvaluate(props.properties.itemLabel);
  if (String(props.dataValue) === String(value)) props.setSelected(label);

  return (
    <MenuItem
      display={
        !!props.search &&
        !label.toLocaleLowerCase().includes(props.search.toLocaleLowerCase())
          ? "none"
          : undefined
      }
      onClick={() => props.onChange(value)}
      backgroundColor={String(props.dataValue) === String(value) && "gray.100"}
    >
      {label}
    </MenuItem>
  );
};

const RadioItem = React.memo(function RadioItem(
  props: {
    item: Row;
    dataValue: string;
    getRadioProps: UseRadioGroupReturn["getRadioProps"];
  } & typeof DropdownListComponent["Component"]["defaultProps"]
) {
  const value = useEvaluate(
    props.properties.itemValue ||
      String(props.item.id) ||
      String(props.source.itemId)
  );
  const { getInputProps, htmlProps, state, getCheckboxProps, getLabelProps } =
    useRadio(props.getRadioProps({ value }));
  const image = useEvaluate(props.properties.itemImage);
  const label = useEvaluate(props.properties.itemLabel);

  return (
    <GridItem
      bg="surface.900"
      rounded="md"
      border="1px"
      borderColor={
        state.isChecked ? props.app.theme.options.primaryColor : "gray.200"
      }
      _hover={
        !state.isChecked && {
          bg: "gray.200",
          borderColor: "gray.300",
        }
      }
    >
      {props.properties.style === "text_choice" && (
        <Radio
          w="full"
          p="1em"
          color={props.properties.style === "text_choice" && "gray.300"}
          colorScheme={
            props.properties.style === "text_choice"
              ? `var(--chakra-colors-qore-primary);`
              : undefined
          }
          {...props.getRadioProps({ value })}
        >
          <Text>{label}</Text>
        </Radio>
      )}
      {props.properties.style === "image_choice" && (
        <Box as="label" cursor="pointer" {...htmlProps}>
          <input {...getInputProps({})} hidden />
          <Stack
            direction={
              props.properties.layout === "horizontal_component"
                ? "row"
                : "column"
            }
            alignItems="center"
            justifyContent={
              props.properties.layout === "vertical_component" && "center"
            }
            spacing="1em"
            w="full"
            p="1em"
            {...getCheckboxProps()}
          >
            {props.properties.style === "image_choice" && (
              <Image
                src={image}
                alt={label}
                fallbackSrc={fallbackImg}
                w="64px"
                h="64px"
                borderRadius="md"
                {...getLabelProps()}
              />
            )}
            <Text display={props.properties.hideLabel && "none"}>{label}</Text>
          </Stack>
        </Box>
      )}
    </GridItem>
  );
});

const CustomRadioGroup = React.memo(function CustomRadioGroup(
  props: {
    isVerticalAlign: boolean;
    listData: Record<string, any>[];
    variableData: {
      onChange: (value: string) => void;
      value: string;
      validation: {
        error: string;
      };
    };
  } & typeof DropdownListComponent["Component"]["defaultProps"]
) {
  const { getRootProps, getRadioProps } = useRadioGroup({
    value: props.variableData.value,
    onChange: props.variableData.onChange,
  });
  return (
    <Grid
      templateColumns={
        props.isVerticalAlign
          ? "repeat(1,1fr)"
          : {
              base: "repeat(1,1fr)",
              md: "repeat(2,1fr)",
              lg: "repeat(3,1fr)",
            }
      }
      gap={2}
      {...getRootProps()}
    >
      {props.listData?.map((item) => (
        <ListItemVariables key={item.id} variables={item}>
          <RadioItem
            item={item}
            dataValue={props.variableData.value}
            getRadioProps={getRadioProps}
            {...props}
          />
        </ListItemVariables>
      ))}
    </Grid>
  );
});

const DropdownListComponent = defineComponent({
  name: "Dropdown List",
  type: "list",
  icon: "SelectInput",
  group: "input",
  display: "block",
  defaultProps: {
    label: "Dropdown List",
    placeholder: "Placeholder here",
    data: {
      inputID: "Dropdown",
      field: { type: "relation", options: {} },
      validation: {
        required: false,
      },
    },
    style: "dropdown",
    itemLabel: "",
    itemImage: "",
    itemValue: "",
    verticalAlignment: false,
    hideLabel: false,
    layout: "vertical_component",
  },
  propDefinition: {
    label: { group: "Design", type: "expression", options: {} },
    placeholder: {
      group: "Design",
      type: "expression",
      options: {},
    },
    data: {
      group: "Design",
      type: "state",
      options: {
        inputID: "Input",
        field: { type: "relation", options: {} },
      },
    },
    style: {
      group: "Design",
      type: "string",
      options: {
        format: "select",
        options: [
          {
            label: "Dropdown",
            value: "dropdown",
          },
          {
            label: "Text Choice",
            value: "text_choice",
          },
          {
            label: "Image Choice",
            value: "image_choice",
          },
        ],
      },
    },
    itemLabel: {
      group: "Design",
      type: "expression",
      options: {},
      section: "dropdown",
    },
    itemImage: {
      group: "Design",
      type: "string",
      options: {
        format: "image",
      },
      section: "dropdown",
      hideOrShownOn: {
        type: "shownOn",
        expression: {
          type: "formula",
          formula: `style == "image_choice"`,
        },
      },
    },
    itemValue: {
      group: "Design",
      type: "expression",
      options: {},
      section: "dropdown",
    },
    verticalAlignment: {
      group: "Design",
      type: "boolean",
      options: {},
      section: "dropdown",
      hideOrShownOn: {
        type: "hideOn",
        expression: {
          type: "formula",
          formula: `style == "dropdown"`,
        },
      },
    },
    hideLabel: {
      group: "Design",
      type: "boolean",
      options: {},
      section: "dropdown",
      hideOrShownOn: {
        type: "shownOn",
        expression: {
          type: "formula",
          formula: `style == "image_choice"`,
        },
      },
    },
    layout: {
      group: "Design",
      section: "dropdown",
      hideOrShownOn: {
        type: "shownOn",
        expression: {
          type: "formula",
          formula: `style == "image_choice"`,
        },
      },
      type: "string",
      options: {
        format: "button",
        options: [
          {
            icon: "IconTopLeft",
            value: "vertical_component",
          },
          {
            icon: "IconLeft",
            value: "horizontal_component",
          },
        ],
      },
    },
  },
  Editor: async () => {
    return {
      data: (props) => {
        const currentPage = RuntimeContext.useSelectState(
          (state) => state.app.maps.page[state.control.page]
        );
        const tables = EditorContext.useSelectState(
          (state) => state.schema.tables
        );
        const views = EditorContext.useSelectState(
          (state) => state.schema.views
        );
        const insights = EditorContext.useSelectState(
          (state) => state.schema.insights
        );
        const tablesAndViews = React.useMemo(
          () =>
            (tables as (TableSchema | ViewSchema | InsightSchema)[]).concat(
              views,
              insights
            ),
          [tables, views, insights]
        );
        const pageView = React.useMemo(
          () =>
            tablesAndViews.find((view) => view.name === currentPage.sourceID),
          [currentPage.sourceID]
        );
        return (
          <props.PropertyEditor.state
            propDef={props.propDef}
            propName={props.propName}
            value={props.value}
            onChange={props.onChange}
            view={pageView}
          />
        );
      },
    };
  },
  Component: (props) => {
    const boxWidth = React.useRef<null | HTMLButtonElement>(null);
    const [selected, setSelected] = React.useState<string>("");
    const [search, setSearch] = React.useState("");
    const sourceHooks = useSourceHooks(props.source);
    const { data: listData = [] } = sourceHooks.useFetchList();

    const title = useEvaluate(props.properties.label);
    const placeholder = useEvaluate(props.properties.placeholder);

    const data = useRuntimeInput<"string">(
      props.properties.data,
      props.data.page.row,
      {
        validate: (value) => {
          if (!value) {
            return { error: "Required field, Please enter the data" };
          }
        },
      }
    );

    const isVerticalAlign = React.useMemo(
      () => props.properties.verticalAlignment,
      [props.properties.verticalAlignment]
    );
    const styleType = React.useMemo(
      () => props.properties.style,
      [props.properties.style]
    );

    return (
      <Box>
        <Text fontSize="lg" pb={2}>
          {title}
        </Text>
        {styleType === "dropdown" && (
          <Menu autoSelect={false}>
            <MenuButton
              as={Button}
              style={{ border: "1px solid rgb(226, 232, 240)" }}
              bgColor={"surface.900"}
              ref={boxWidth}
              textAlign="left"
              borderRadius="md"
              variant="outline"
              rightIcon={<ChevronDown />}
              width="100%"
              _focus={{ shadow: "0px" }}
              fontWeight="normal"
              data-test="dropdown-list-menu-button"
            >
              {data.value ? selected : placeholder}
            </MenuButton>
            <MenuList
              border="0px"
              width={boxWidth?.current?.offsetWidth}
              zIndex={3}
            >
              <Box paddingX="0.8rem">
                <Input
                  value={search}
                  onChange={(e) => setSearch(e.target.value)}
                  placeholder="Search..."
                  mb={2}
                  autoFocus
                />
              </Box>
              <Box maxHeight="11.5rem" overflowY="auto">
                <Text color="gray" py={2} px={4} style={{ cursor: "default" }}>
                  {placeholder}
                </Text>
                {listData?.map((item) => (
                  <ListItemVariables key={item.id} variables={item}>
                    <Option
                      item={item}
                      onChange={data.onChange}
                      dataValue={data.value}
                      setSelected={setSelected}
                      {...props}
                      data-test="dropdown-list-option"
                      search={search}
                    />
                  </ListItemVariables>
                ))}
              </Box>
            </MenuList>
          </Menu>
        )}
        {(styleType === "text_choice" || styleType === "image_choice") && (
          <CustomRadioGroup
            variableData={data}
            listData={listData}
            isVerticalAlign={isVerticalAlign}
            {...props}
          />
        )}
        {!!data.validation.error && (
          <Text fontSize="sm" color="red" data-test="text-input-error">
            {data.validation.error}
          </Text>
        )}
      </Box>
    );
  },
});

export default DropdownListComponent;
